import QtQuick 2.12
import Process 0.1
import "../Television"

Item {
    property alias accessPoints: wifiAccessPoints
    property alias scanning: wifiScanProcess.running
    property bool debug: false

    signal scanComplete()

    function startScan() {
        wifiScanProcess.start()
    }

    function debugLog(string) {
        if(root.debug)
            print("WifiScan: " + string)
    }

    ListModel {
        id: wifiAccessPoints
    }

    Process {
        id: wifiScanProcess
        property variant cell: ({})
        property bool running: processState == Process.Running

        program: "scripts/scan_wifi.sh"
        arguments: [main.wifiAdapter]

        onStarted: wifiAccessPoints.clear()

        onFinished: {
            if(wifiScanProcess.cell) {
                if(cell.essid && findESSID(cell.essid) < 0)
                    wifiAccessPoints.append(wifiScanProcess.cell)
            }
            scanComplete()
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "wifi scan error output: " + output)
        }

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog(output)
            parseIWScanOutput(output)
//            parseScanOutput(output)
        }

        function findESSID(ID) {
            for(var i=0 ; i < wifiAccessPoints.count ; i++)
                if(ID === wifiAccessPoints.get(i).essid)
                    return i
            return -1
        }

        function parseIWScanOutput(output) {
            //"MAC, SSID, Signal, Encryption, Auth";
            var lines = output.split("\n")
            for(var i=0 ; i < lines.length ; i++) {
                var line = lines[i]
                var cell = line.split(',')
                var essid = cell[1]
                var found = false
                if(essid) {
                    for (var j=0; j< wifiAccessPoints.count; j++) {
                        if(wifiAccessPoints.get(j).essid === essid) {
                            // already added essid
                            found = true
                            break;
                        }
                    }

                    if(!found) {
                        wifiAccessPoints.append({essid: cell[1], interface: main.wifiAdapter, mac: cell[0],
                                                    encrypted:cell[3]?true:false, quality: Math.abs(cell[2]),
                                                    auth: cell[4], password: "" })
                    }
                }

            }
            console.log(wifiAccessPoints.count + " access points.")

        }

        function parseScanOutput(output) {
            var cell = wifiScanProcess.cell
            var lines = output.split("\n")

            for(var i=0 ; i < lines.length ; i++) {
                var line = lines[i]

                if(line.match("Cell")) {
                    if(cell.essid && findESSID(cell.essid) < 0 ) {
                        debugLog("---> append:" + cell.essid)
                        wifiAccessPoints.append({essid: cell.essid, interface: cell.interface, mac: cell.mac,
                                                    encrypted:cell.encrypted, quality: cell.quality,
                                                    auth: cell.auth, password: "" })
                        debugLog(wifiAccessPoints.count + " access points.")
                        cell = {}
                    }
                    var addr = line.split("Address: ")[1]
                    debugLog(">> addr:"+addr)
                    cell.mac = addr
                }
                else if(line.match("ESSID:")){
                   var essid = line.split("ESSID:")[1]
                    essid = essid.split('"')[1]
                    debugLog(">> ESSID:" + essid)
                    cell.essid = essid
                    cell.interface = main.wifiAdapter
                }
                else if(line.match("Encryption key:")) {
                    var encrypted = line.split("Encryption key:")[1]
                    if(encrypted == "on")
                        cell.encrypted = true
                    else
                        cell.encrypted = false
                    debugLog(">> Encrypted:" + cell.encrypted)
                }
                else if(line.match("Quality=")) {
                    var quality = line.split("Quality=")[1]
                    quality = quality.split(" ")[0]
                    debugLog(">> Quality:"+quality)
                    cell.quality = quality
                }
                else if(line.match("Authentication Suites")) {
                    var auth = line.split(":")[1]
                    auth = auth.trim()
                    debugLog(">> auth:"+auth)
                    cell.auth = auth
                }
            }
            wifiScanProcess.cell = cell
        } // end parseScanOutput()
    } // end Process {}
}
