import QtQuick 2.4
import QtQuick.Layouts 1.12
import ".."
import Components 1.0
import Television 1.0
import Process 0.1

Item {
    id:root

    property bool isSingleHDMI: options.singlePortHDMI
    property bool viewShowing: false
    property bool fullscreen: false
    property string mirrorDevice: ""

    property bool isMenuDisplayed: true
    property bool isMenuMoving: false

    signal stopped()
    signal menuButtonPressed()
    signal action()

    Process {
        id: selectPort
        program: "scripts/select_port.sh"
        arguments: ["s3"]
        onStarted: {
            console.log("Select MUX port " + root.mirrorDevice)
        }
        onFinished: {
            hdmiView.play()
        }
    }

    function play() {
        if(isSingleHDMI) {
            hdmiView.play()
        } else {
            selectPort.arguments = [root.mirrorDevice]
            selectPort.start()
        }
    }

    function shown() {
        root.viewShowing = true
        if(isSingleHDMI) {
            play()
        } else {
            if(root.mirrorDevice === "")
                menu_view.show()
        }
    }

    function hidden() {
        root.viewShowing = false
        menu_view.hide()
        hdmiView.stop();
    }

    function stop() {
        root.stopped()
        menu_view.hide()
        hdmiView.stop();
        root.mirrorDevice = ""
    }

    Component.onCompleted: {
    }

    onMirrorDeviceChanged: {
        if(root.mirrorDevice !== "") {
            play()
        }
    }

    Connections {
        target: workoutScreen
        onWorkoutStopped: root.stop()
    }

    Item {
        id: hdmiView

        function play() {
            tv.viewParent = hdmiView
            tv.changeParent(hdmiView)
            tv.mrl = "v4l2:///dev/video1:width=1920:height=1080"

            tv.on = true
            tv.muted = !audioManager.tvAllowed
            audioManager.tvRequest()
        }

        function stop() {
            tv.stop()
        }

        Rectangle {
            id:tvNotPlaying
            anchors.fill: parent
            color: "black"
            visible: true

            Text {
                anchors.centerIn: parent
                color: "white"
                visible: false
                text: qsTr("Connect a device") + application.emptyString
            }
        }

        anchors.fill: parent
    }

    ListModel {
        id: hdmiModel
        ListElement {
            label: QT_TR_NOOP("ANDROID");
            device: "s2";
        }
        ListElement {
            label: QT_TR_NOOP("IPHONE");
            device: "s3";
        }
    }

    ModalDialog {
        id: menu_view

        Item {
            anchors.fill: parent
            z: parent.z + 1

            TextLabel {
                id: titleText
                text: qsTr("Select Device")
                font.pixelSize: 30 * main.vScaleFactor
                anchors {
                    top: parent.top
                    topMargin: 20 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                }
            }

            ListView {
                anchors {
                    top: titleText.bottom
                    topMargin: 20 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                    bottom: warningText.top
                }

                width: 0.5*parent.width
                orientation: ListView.Horizontal
                model: hdmiModel
                spacing: 20 * main.hScaleFactor

                delegate: Component {
                    StandardButton {
                        buttonText: model.label
                        onClicked: {
                            menu_view.hide()
                            root.mirrorDevice = model.device
                            root.play()
                        }
                    }
                }
            }

            TextLabel {
                id: warningText

                width: 0.8 * parent.width
                height: 99 * main.vScaleFactor

                text: qsTr("Not all apps or phones may be supported by this feature.")  + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 35 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 30 * main.vScaleFactor
            }
        }

        Component.onCompleted: hide()
    }
}
